#  send command to Dataplot and expect certain strings

proc SendDP {cmd} {

   global nplot env
   
   set dirimg /home/lipman/public_html/dataplot

   set convertcmd /usr/local/apps/ImageMagick.sgi/bin/convert
#   set convertcmd /usr/freeware/bin/convert

   set ext gif
#   set ext jpg

   set notdone 1
   exp_send "$cmd\n"
   while {$notdone} {
      expect {
	 "MORE...?" {exp_send "\n"}
	 "THIS IS AN EXIT FROM DATAPLOT" {set notdone 0}
	 -re "\r\n>" {set notdone 0}
      }

      if {[info exists expect_out(buffer)]} {
         set output $expect_out(buffer)
         
#  check for a ` indicating a plot was generated
         set pos [string first "`" $output]

#  plot exists, send DEVICE 3 CLOSE, to close postscript file
         if {$pos != -1} {
            set output [string range $output 0 \
               [string first \n [string range $output 2 end]]]
            PrintOutput $output
            exp_send "DEVICE 3 CLOSE\n"
            expect -re "\r\n>"

#  set plot file name
            incr nplot
            set pid [pid]
            set fname dppl2f$nplot$pid

#  scale font size
            if {[catch {exec sed -e "s/24 scalefont/46 scalefont/" /tmp/dppl2f.dat > /tmp/$fname.ps} err]} {
               puts </PRE>
               set msg "WARNING: scaling Postscript font size"
               ErrMsg $msg $err
               puts <PRE>
               puts $err\n
               exec cp /tmp/dppl2f.dat /tmp/$fname.ps
            }
            
#  check that scaled Postscript file exists
            if {[catch {exec ls -l /tmp/$fname.ps} err]} {            
               puts </PRE>
               set msg "ERROR: scaled Postscript file not found"
               ErrMsg $msg $err
               puts <PRE>
               puts $err\n
               exec cp /tmp/dppl2f.dat /tmp/$fname.ps
            }

#  convert ps to $ext (gif or jpg)
#           puts [exec ls -l /tmp/$fname.ps]
#           puts [exec env | sort]
           set env(PATH) $env(PATH):/itl/links/generic/bin
           if [catch {exec $convertcmd -rotate 90. -crop 0x0 /tmp/$fname.ps $dirimg/$fname.$ext} err] {
               puts </PRE>
               set msg "ERROR: converting Dataplot Postscript file to $ext image"
#               ErrMsg $msg $err
               puts <PRE>
               puts $err\n

#  conversion successful, link to image and display
            } else {
#               cgi_imglink PLOT$nplot$pid file:$dirimg/$fname.$ext alt=\[DataplotGraph_$nplot\] 
#               cgi_imglink PLOT$nplot$pid http://math.nist.gov/mcsd/Staff/RLipman/dataplot/$fname.$ext alt=\[DataplotGraph_$nplot\] 
               cgi_imglink PLOT$nplot$pid http://vislab.cam.nist.gov/~lipman/dataplot/$fname.$ext alt=\[DataplotGraph_$nplot\] 
               exec chmod go+w $dirimg/$fname.$ext

#  display image
               cgi_table "border=2 cellpadding=10" {
        	  cgi_table_row {cgi_table_data {puts [cgi_imglink PLOT$nplot$pid]}}
               }
               cgi_br
            }

#  remove postscript file
            exec rm /tmp/$fname.ps

#  print command text output
         } else {
            PrintOutput $output
         }
         
         if {$cmd == "EXIT"} {set notdone 0}

#  error sending command
      } else {
#	 puts </PRE>
#         ErrMsg "ERROR: executing command"
#	 puts <PRE>
      }
   }
}   

################################################################################

#  print output on web page

proc PrintOutput {output} {

   set boundary "ThisRandomString"
   set notdone 1
   set output [split $output \n]
   while {$notdone} {
      set line [lindex $output 1]
      set output [lrange $output 2 end]
      if {$line != ""} {set notdone 0}
   }

   set output [join $output \n]
   regsub -all { ERROR} $output [cgi_bold [cgi_font "color=#ff0000" " ERROR"]] output
   set output [string trimright $output "\n>"]
   set output [string trimleft $output "\r"]
   set line [string trimright $line "\r"]

   cgi_http_head {
      cgi_content_type "multipart/x-mixed-replace;boundary=$boundary"
      puts \n--$boundary
      cgi_content_type
   }
#   puts [cgi_font size=-1 [cgi_bold $line]]
#   if {$output != ""} {puts [cgi_font size=-1 $output]}
   puts [cgi_bold $line]
   if {$output != ""} {puts $output}
}

################################################################################

#  execute command for web page

proc ExecCmd {cmd {arg1 ""} {arg2 ""}} {

   global VALID_CMDS

   set cmdout ""

#  limit the set of commands (VALID_CMDS) that can be run (defined in site.tcl)
   if {[info exists VALID_CMDS]} {
      set ok 0
      foreach item $VALID_CMDS {if {$item == $cmd} {set ok 1}}
   } else {
      set ok 1
   }

   if {$ok} {
      if {[FindCmd $cmd]} {
	 if {[catch {eval exec $cmd $arg1 $arg2} cmdout]} {
	    ErrMsg "ERROR: executing $cmd $arg1 $arg2"
	 }
	 cgi_preformatted {puts $cmdout}
      }
   } else {
      ErrMsg "ERROR: trying to run command other than: $VALID_CMDS"
   }
   
   return $cmdout
}

################################################################################

#  print error message on web page and mail

proc ErrMsg {msg {err ""}} {

   if {$err != ""} {
      cgi_mail_start [cgi_admin_mail_addr]
      cgi_mail_add "Subject: [cgi_name] $msg"
      cgi_mail_add "[cgi_name] $msg"
      cgi_mail_add "$err"
      cgi_mail_end
   } 

   regsub -all {ERROR:}   $msg [font "color=#ff0000" ERROR:]   msg
   regsub -all {WARNING:} $msg [font "color=#ff0000" WARNING:] msg
   cgi_p [cgi_bold $msg]
}

################################################################################

#  Look for commands in directories of PATH environment variable
#  1 - found, 0 - not found

proc FindCmd {cmd} {
   global env

   if {$env(PATH) != ""} {
      foreach dir [split $env(PATH) ":"] {
         set file [file join $dir $cmd]
         if {[file exists $file]} {
	    if {[file isfile $file] && [file executable $file]} {return 1}
         }
      }
      ErrMsg "ERROR: $cmd command not found"
      return 0
   } else {
      return 1
   }
}
